#include "TArtBeam.h"
#include "TArtBeamPara.h"
#include "TArtCore.h"
#include "TArtRIPS.h"
#include "TArtTOF.h"
#include "TArtIC.h"
#include "TArtBigRIPSParameters.h"

#include <TList.h>
#include <TMath.h>

void TArtBeamArray::LoadSetup(TArtBigRIPSParameters *para) 
{

  Clear();
  TList *listOfBeamPara = (TList *)para->GetListOfBeamPara();
  TIter next(listOfBeamPara);
  TArtBeamPara *p;
  while(p = (TArtBeamPara*)next()) {
    Add(new TArtBeam(p));
  }

}

//__________________________________________________________
void TArtBeamArray::LoadMap(TArtRIPSArray *rips,TArtTOFArray *tof,TArtICArray *ic)
{
  for(int i=0;i<GetEntries();i++)
    ((TArtBeam*)At(i))->LoadMap(rips,tof,ic);
  loadmap = true;
}

//__________________________________________________________
void TArtBeamArray::ClearData()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtBeam*)At(i))->ClearData();
}

//__________________________________________________________
void TArtBeamArray::RecoAoQZ()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtBeam*)At(i))->RecoAoQZ();
}

//__________________________________________________________
TArtBeam::TArtBeam(TArtBeamPara *p) : para(p)
{
  clight = 299.792458; // mm/nsec
  mnucleon = 931.49432; // MeV
}

//__________________________________________________________
TArtBeam::~TArtBeam() 
{
}

//__________________________________________________________
void TArtBeam::LoadMap(TArtRIPSArray *rips_array, TArtTOFArray *tof_array, TArtICArray *ic_array)
{

  forward_rips = NULL;
  backward_rips = NULL;
  tof = NULL;
  ic = NULL;

  TArtRIPSPara * frips_para = (TArtRIPSPara *)para->GetForwardRIPS();
  TArtRIPSPara * brips_para = (TArtRIPSPara *)para->GetBackwardRIPS();
  TArtTOFPara  * tof_para = (TArtTOFPara *)para->GetTOF();
  TString * icname = para->GetDeltaEName();

  Int_t fripsid = frips_para->GetID();
  Int_t bripsid = brips_para->GetID();
  for(int i=0;i<rips_array->GetEntries();i++){
    TArtRIPS * r = (TArtRIPS *)rips_array->At(i);
    if( r->GetID() == fripsid ){
      if( NULL == forward_rips )
	forward_rips = r;
      else
	TArtCore::Info(__FILE__,"multiple forward rips are defined!: fpl%d - fpl%d",frips_para->GetForwardFpl(),frips_para->GetBackwardFpl());
    }
    if( r->GetID() == bripsid ){
      if( NULL == backward_rips )
	backward_rips = r;
      else
	TArtCore::Info(__FILE__,"multiple backward rips are defined!: fpl%d - fpl%d",brips_para->GetForwardFpl(),brips_para->GetBackwardFpl());
    }
  }

  Int_t tofid = tof_para->GetID();
  for(int i=0;i<tof_array->GetEntries();i++){
    TArtTOF * t = (TArtTOF *)tof_array->At(i);
    if( t->GetID() == tofid ){
      if( NULL == tof )
	tof = t;
      else
	TArtCore::Info(__FILE__,"multiple tof are defined!: %s - %s",tof_para->GetForwardPlName()->Data(),tof_para->GetBackwardPlName()->Data());
    }
  }

  for(int i=0;i<ic_array->GetEntries();i++){
    TArtIC * c = (TArtIC *)ic_array->At(i);
    if( strcmp(c->GetDetectorName()->Data(), icname->Data() ) == 0 ){
      if( NULL == ic )
	ic = c;
      else
	TArtCore::Info(__FILE__,"multiple ic are defined!: %s",icname->Data());
    }
  }

  if(!( forward_rips && backward_rips && tof && ic ))
    TArtCore::Info(__FILE__,"failed to make map for beam ic:%s",icname->Data());

}

//__________________________________________________________
void TArtBeam::ClearData()
{
  aoq = 0;
  zet = 0;
  beta = 0;
}

//__________________________________________________________
void TArtBeam::RecoAoQZ()
{

  Double_t brho = 0;
  beta = 0;
  Int_t fpl_ic = ic->GetFpl();

  if(para->GetNumRIPS() == 1){
    brho = forward_rips->GetBrho();
    beta = tof->GetBeta();
    Double_t gamma = tof->GetGamma();
    aoq = brho * clight / mnucleon / beta / gamma;
  }
  else if(para->GetNumRIPS() == 2){
    Double_t brho1 = forward_rips->GetBrho();
    Double_t brho2 = backward_rips->GetBrho();
    Double_t ftime = tof->GetTOF();
    Double_t fl1 = forward_rips->GetLength();
    Double_t fl2 = backward_rips->GetLength();
    Double_t rbeta1;
    Double_t rbeta2;

    Double_t alpha  = brho2/brho1;
    Double_t a1     = TMath::Sqrt(alpha*alpha * clight*clight * ftime*ftime
				  + (TMath::Power(alpha,4) - alpha*alpha) * fl1*fl1
				  + (1 - alpha*alpha) * fl2*fl2);
  
    rbeta1 = ( a1*fl1 + fl2*clight*ftime ) / ( a1*clight*ftime + (1-alpha*alpha)*fl1*fl2 );
    Double_t gammaf = 1/sqrt(1-pow(rbeta1,2));
    rbeta2  = ( a1*fl1 + fl2*clight*ftime )/ ( clight*clight * ftime*ftime + (alpha*alpha-1)*fl1*fl1 );
    Double_t gammab = 1/sqrt(1-pow(rbeta2,2));
    aoq = brho1*clight/mnucleon/rbeta1/gammaf; // should be same as brho2/beta2/gamma2

    forward_rips->SetBeta(rbeta1);
    backward_rips->SetBeta(rbeta2);

    if( backward_rips->GetForwardFpl() < fpl_ic )
      beta = rbeta2;
    else if(forward_rips->GetForwardFpl() < fpl_ic )
      beta = rbeta1;
    else
      TArtCore::Info(__FILE__,"ic looks out of range!!: %d",fpl_ic);

    //TArtCore::Info(__FILE__,"brho1:%f, brho2:%f, aoq1: %f, aoq2: %f, org-beta:%f, beta1: %f, beta2:%f",brho1,brho2,aoq,brho2*clight/mnucleon/rbeta2/gammab,(fl1+fl2)/ftime/clight,rbeta1,rbeta2);

  }

  zet = ic->RecoZet(beta);

}

//__________________________________________________________
Int_t TArtBeam::GetID() {
  return para->GetID();
}
